@@ -75,7 +75,7 @@ def _tiecorrect(ranks):
     idx = np.sort(idx, axis=0)
     cnt = np.diff(idx, axis=0).astype(np.float64)
 
-    return 1.0 - (cnt ** 3 - cnt).sum(axis=0) / (size ** 3 - size)
+    return 1.0 - (cnt**3 - cnt).sum(axis=0) / (size**3 - size)
 
 
 class _RankGenes:
@@ -670,9 +670,10 @@ def filter_rank_genes_groups(
     min_in_group_fraction=0.25,
     min_fold_change=1,
     max_out_group_fraction=0.5,
+    compare_abs=False,
 ) -> None:
     """\
-    Filters out genes based on fold change and fraction of genes expressing the
+    Filters out genes based on log fold change and fraction of genes expressing the
     gene within and outside the `groupby` categories.
 
     See :func:`~scanpy.tl.rank_genes_groups`.
@@ -693,6 +694,8 @@ def filter_rank_genes_groups(
     min_in_group_fraction
     min_fold_change
     max_out_group_fraction
+    compare_abs
+        If `True`, compare absolute values of log fold change with `min_fold_change`.
 
     Returns
     -------
@@ -794,6 +797,8 @@ def filter_rank_genes_groups(
                 / (expm1_func(mean_out_cluster) + 1e-9)
             )
 
+    if compare_abs:
+        fold_change_matrix = fold_change_matrix.abs()
     # filter original_matrix
     gene_names = gene_names[
         (fraction_in_cluster_matrix > min_in_group_fraction)
